//! Internal errors
use super::*;
use std::{
    fmt,
    error,
    net::SocketAddr,
};

#[derive(Debug)]
#[non_exhaustive]
pub enum Error {
    Denied(SocketAddr, bool),
    Unknown,
}

impl Error
{
    /// Print this error as a warning
    #[inline(never)] #[cold] pub fn warn(self) -> Self
    {
	warn!("{}", self);
	self
    }
    
    /// Print this error as info
    #[inline(never)] #[cold] pub fn info(self) -> Self
    {
	info!("{}", self);
	self
    }
}

impl error::Error for Error{}
impl fmt::Display for Error
{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result
    {
	match self {
	    Self::Denied(sock, true) => write!(f, "denied connection (explicit): {}", sock),
	    Self::Denied(sock, _) => write!(f, "denied connection (implicit): {}", sock),
	    _ => write!(f, "unknown error"),
	}
    }
}
